import yaml
from mock import patch
from pytest import raises

from kiwi_boxed_plugin.plugin_config import PluginConfig
from kiwi_boxed_plugin.exceptions import KiwiBoxPluginConfigError


class TestPluginConfig:
    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    def setup(self, mock_get_plugin_config_file):
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin.yml'
        self.plugin_config = PluginConfig()

    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    def setup_method(self, cls, mock_get_plugin_config_file):
        self.setup()

    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    def test_invalid_config(self, mock_get_plugin_config_file):
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin_invalid.yml'
        with raises(KiwiBoxPluginConfigError):
            PluginConfig()

    def test_get_config(self):
        print(self.plugin_config.get_config())
        assert self.plugin_config.get_config() == [
            {
                'name': 'suse',
                'mem_mb': 8096,
                'processors': 4,
                'console': 'hvc0',
                'arch': [
                    {
                        'name': 'x86_64',
                        'cmdline': ['root=/dev/vda1', 'rd.plymouth=0'],
                        'source':
                            'obs://Virtualization:Appliances:SelfContained:'
                            'suse/images',
                        'packages_file':
                            'SUSE-Box.x86_64-1.42.1-System-BuildBox.report',
                        'boxfiles': [
                            'SUSE-Box.x86_64-1.42.1-Kernel-BuildBox.tar.xz',
                            'SUSE-Box.x86_64-1.42.1-System-BuildBox.qcow2'
                        ],
                        'use_initrd': True
                    }
                ]
            },
            {
                'name': 'universal',
                'mem_mb': 8096,
                'processors': 4,
                'console': 'hvc0',
                'arch': [
                    {
                        'name': 'x86_64',
                        'cmdline': [
                            'root=/dev/vda3', 'rd.plymouth=0', 'selinux=0'
                        ],
                        'source':
                            'obs://Virtualization:Appliances:SelfContained:'
                            'universal/images',
                        'packages_file':
                            'Universal-Box.x86_64-1.1.2-System-BuildBox.report',
                        'boxfiles': [
                            'Universal-Box.x86_64-1.1.2-Kernel-BuildBox.tar.xz',
                            'Universal-Box.x86_64-1.1.2-System-BuildBox.qcow2'
                        ],
                        'use_initrd': True
                    },
                    {
                        'name': 'aarch64',
                        'cmdline': [
                            'root=/dev/vda2', 'rd.plymouth=0', 'selinux=0'
                        ],
                        'source':
                            'obs://Virtualization:Appliances:SelfContained:'
                            'universal/images',
                        'packages_file': 'Universal-Box.'
                            'aarch64-1.1.2-System-BuildBox.report',
                        'boxfiles': [
                            'Universal-Box.aarch64-1.1.2-'
                            'Kernel-BuildBox.tar.xz',
                            'Universal-Box.aarch64-1.1.2-'
                            'System-BuildBox.qcow2'
                        ],
                        'use_initrd': True
                    }
                ]
            }
        ]

    def test_dump_config(self):
        assert self.plugin_config.dump_config() == yaml.dump(
            self.plugin_config.get_config()
        )

    @patch('yaml.safe_load')
    def test_setup_raises(self, mock_yaml_safe_load):
        mock_yaml_safe_load.side_effect = Exception
        with raises(KiwiBoxPluginConfigError):
            PluginConfig()



from mock import (
    Mock, patch, call
)

from kiwi_boxed_plugin.utils.dir_files import DirFiles


class TestDirFiles:
    @patch('kiwi_boxed_plugin.utils.dir_files.Path')
    def setup(self, mock_Path):
        self.dir_manager = DirFiles('box_dir')
        mock_Path.wipe.assert_called_once_with('box_dir.tmp')

    @patch('kiwi_boxed_plugin.utils.dir_files.Path')
    def setup_method(self, cls, mock_Path):
        self.setup()

    @patch('kiwi_boxed_plugin.utils.dir_files.NamedTemporaryFile')
    def test_register(self, mock_NamedTemporaryFile):
        tmpfile = Mock()
        tmpfile.name = 'tmp_a'
        mock_NamedTemporaryFile.return_value = tmpfile
        self.dir_manager.register('/some/path/to/file_a')
        assert self.dir_manager.collection == {
            'file_a': 'tmp_a'
        }

    @patch('kiwi_boxed_plugin.utils.dir_files.Path')
    @patch('kiwi_boxed_plugin.utils.dir_files.Command.run')
    def test_commit(self, mock_Command_run, mock_Path):
        self.dir_manager.collection = {
            'file_a': 'tmp_a'
        }
        self.dir_manager.commit()
        mock_Path.create.assert_called_once_with('box_dir.tmp')
        assert mock_Command_run.call_args_list == [
            call(
                ['bash', '-c', 'cp -a box_dir/* box_dir.tmp'],
                raise_on_error=False
            ),
            call(['mv', 'tmp_a', 'box_dir.tmp/file_a']),
            call(['mv', 'box_dir', 'box_dir.wipe']),
            call(['mv', 'box_dir.tmp', 'box_dir']),
            call(['rm', '-rf', 'box_dir.wipe'])
        ]



import io
from mock import (
    Mock, MagicMock, patch
)

from kiwi_boxed_plugin.utils.fetch_files import FetchFiles


class TestFetchFiles:
    def setup(self):
        self.fetcher = FetchFiles()

    def setup_method(self, cls):
        self.setup()

    @patch('kiwi_boxed_plugin.utils.fetch_files.requests')
    @patch('kiwi_boxed_plugin.utils.fetch_files.progressbar')
    def test_wget(self, mock_progressbar, mock_requests):
        response = Mock()
        response.headers.get.return_value = 4
        response.iter_content.return_value = [b'data']
        mock_requests.request.return_value = response
        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value
            self.fetcher.wget('http://foo', 'bar')
            mock_open.assert_called_once_with(
                'bar', 'wb'
            )
            file_handle.write.assert_called_once_with(b'data')
            mock_progressbar.ProgressBar.assert_called_once_with(
                maxval=4,
                widgets=[
                    'Loading: ',
                    mock_progressbar.Percentage.return_value, ' ',
                    mock_progressbar.Bar(marker='#', left='[', right=']'), ' ',
                    mock_progressbar.ETA.return_value, ' ',
                    mock_progressbar.FileTransferSpeed.return_value
                ]
            )



from mock import patch
from pytest import raises

from kiwi_boxed_plugin.box_config import BoxConfig
from kiwi_boxed_plugin.exceptions import (
    KiwiBoxPluginConfigError,
    KiwiBoxPluginBoxNameError,
    KiwiBoxPluginArchNotFoundError
)


class TestBoxConfig:
    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    @patch('platform.machine')
    def setup(self, mock_platform_machine, mock_get_plugin_config_file):
        mock_platform_machine.return_value = 'x86_64'
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin.yml'
        self.box_config = BoxConfig('suse')

    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    @patch('platform.machine')
    def setup_method(
        self, cls, mock_platform_machine, mock_get_plugin_config_file
    ):
        self.setup()

    @patch('yaml.safe_load')
    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    def test_setup_raises_on_load_config(
        self, mock_get_plugin_config_file, mock_yaml_safe_load
    ):
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin.yml'
        mock_yaml_safe_load.side_effect = Exception
        with raises(KiwiBoxPluginConfigError):
            BoxConfig('suse')

    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    def test_setup_raises_on_unsupported_arch(
        self, mock_get_plugin_config_file
    ):
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin.yml'
        with raises(KiwiBoxPluginArchNotFoundError):
            BoxConfig('suse', 'artificial_arch')

    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    def test_setup_raises_box_not_found(self, mock_get_plugin_config_file):
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin.yml'
        with raises(KiwiBoxPluginBoxNameError):
            self.box_config = BoxConfig('foo', 'x86_64')

    def test_get_box_arch(self):
        assert self.box_config.get_box_arch() == 'x86_64'

    def test_get_box_memory_mbytes(self):
        assert self.box_config.get_box_memory_mbytes() == 8096

    def test_get_box_console(self):
        assert self.box_config.get_box_console() == 'hvc0'

    def test_get_box_kernel_cmdline(self):
        assert self.box_config.get_box_kernel_cmdline() == \
            'root=/dev/vda1 rd.plymouth=0'

    def test_get_box_source(self):
        assert self.box_config.get_box_source() == \
            'obs://Virtualization:Appliances:SelfContained:suse/images'

    def test_get_box_packages_file(self):
        assert self.box_config.get_box_packages_file() == \
            'SUSE-Box.x86_64-1.42.1-System-BuildBox.report'

    def test_get_box_packages_shasum_file(self):
        assert self.box_config.get_box_packages_shasum_file() == \
            'SUSE-Box.x86_64-1.42.1-System-BuildBox.report.sha256'

    def test_get_box_files(self):
        assert self.box_config.get_box_files() == [
            'SUSE-Box.x86_64-1.42.1-Kernel-BuildBox.tar.xz',
            'SUSE-Box.x86_64-1.42.1-System-BuildBox.qcow2'
        ]



import logging
from mock import (
    patch, Mock, call
)
from pytest import raises

from kiwi_boxed_plugin.box_build import BoxBuild
import kiwi_boxed_plugin.defaults as defaults

from kiwi_boxed_plugin.exceptions import (
    KiwiBoxPluginVirtioFsError,
    KiwiBoxPluginQEMUBinaryNotFound,
    KiwiBoxPluginSSHPortInvalid,
    KiwiError
)

log = logging.getLogger('kiwi')
log.setLevel('INFO')


class TestBoxBuild:
    @patch('kiwi_boxed_plugin.box_build.BoxDownload')
    def setup(self, mock_BoxDownload):
        self.box = Mock()
        self.vm_setup = Mock()
        self.vm_setup.kernel = 'kernel'
        self.vm_setup.append = 'append'
        self.vm_setup.system = 'system'
        self.vm_setup.initrd = 'initrd'
        self.vm_setup.ram = 4096
        self.vm_setup.smp = 4
        self.box.fetch.return_value = self.vm_setup
        mock_BoxDownload.return_value = self.box
        self.build = BoxBuild(
            boxname='suse', arch='x86_64'
        )
        self.build_arm = BoxBuild(
            boxname='universal', arch='aarch64',
            machine='virt', cpu='cortex-a57'
        )

    @patch('kiwi_boxed_plugin.box_build.BoxDownload')
    def setup_method(self, cls, mock_BoxDownload):
        self.setup()

    @patch('os.environ')
    @patch('os.system')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.box_build.Path.which')
    def test_raises_on_kiwi_error(
        self, mock_path_which, mock_path_create,
        mock_os_system, mock_os_environ
    ):
        mock_path_which.return_value = 'qemu-system-x86_64'
        with raises(KiwiError):
            self.build.run(
                [
                    '--type', 'oem', 'system', 'build',
                    '--description', 'desc', '--target-dir', '../data/target'
                ]
            )

    @patch('os.environ')
    @patch('os.system')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.box_build.Path.which')
    def test_raises_on_invalid_ssh_port(
        self, mock_path_which, mock_path_create,
        mock_os_system, mock_os_environ
    ):
        mock_path_which.return_value = 'qemu-system-x86_64'
        self.build.ssh_port = 'bogus'
        with raises(KiwiBoxPluginSSHPortInvalid):
            self.build.run(
                [
                    '--debug', '--type', 'oem', 'system', 'build',
                    '--description', 'desc', '--target-dir', 'target'
                ]
            )

    @patch('os.environ')
    @patch('os.system')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.box_build.Path.which')
    def test_run_with_9p_sharing(
        self, mock_path_which, mock_path_create,
        mock_os_system, mock_os_environ
    ):
        mock_path_which.return_value = 'qemu-system-x86_64'
        self.build.run(
            [
                '--debug', '--type', 'oem', 'system', 'build',
                '--description', 'desc', '--target-dir', 'target'
            ],
            keep_open=True,
            kiwi_version='9.22.1',
            custom_shared_path='/var/tmp/repos'
        )
        mock_path_create.assert_called_once_with('target')
        mock_os_system.assert_called_once_with(
            'qemu-system-x86_64 '
            '-m 4096 '
            '-accel accel=kvm '
            '-cpu host '
            '-nographic '
            '-nodefaults '
            '-snapshot '
            '-kernel kernel '
            '-append "append kiwi=\\"--debug --type oem system build\\"'
            ' kiwi-no-halt kiwi_version=_9.22.1_'
            ' custom_mount=_/var/tmp/repos_'
            ' sharing_backend=_9p_" '
            '-drive file=system,if=virtio,driver=qcow2,cache=off,snapshot=on '
            '-netdev user,id=user0,hostfwd=tcp::10022-:22 '
            '-device virtio-net-pci,netdev=user0 '
            '-device virtio-serial '
            '-chardev stdio,id=virtiocon0 '
            '-device virtconsole,chardev=virtiocon0 '
            '-fsdev local,security_model=mapped,id=fsdev0,path=desc '
            '-device virtio-9p-pci,id=fs0,fsdev=fsdev0,mount_tag='
            'kiwidescription '
            '-fsdev local,security_model=mapped,id=fsdev1,path=target '
            '-device virtio-9p-pci,id=fs1,fsdev=fsdev1,mount_tag='
            'kiwibundle '
            '-fsdev local,security_model=mapped,id=fsdev2,path=/var/tmp/repos '
            '-device virtio-9p-pci,id=fs2,fsdev=fsdev2,mount_tag='
            'custompath '
            '-initrd initrd '
            '-smp 4'
        )

    @patch('os.environ')
    @patch('os.system')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.box_build.Path.which')
    def test_run_cross_arch_aarch64_on_x86_64(
        self, mock_path_which, mock_path_create,
        mock_os_system, mock_os_environ
    ):
        mock_path_which.return_value = 'qemu-system-aarch64'
        self.build_arm.run(
            [
                '--type', 'oem', 'system', 'build',
                '--description', 'desc', '--target-dir', 'target'
            ],
            keep_open=True,
            kiwi_version='9.22.1',
            custom_shared_path='/var/tmp/repos'
        )
        mock_path_create.assert_called_once_with('target')
        mock_os_system.assert_called_once_with(
            'qemu-system-aarch64 '
            '-m 4096 '
            '-machine virt '
            '-accel accel=kvm '
            '-cpu cortex-a57 '
            '-nographic '
            '-nodefaults '
            '-snapshot '
            '-kernel kernel '
            '-append "append kiwi=\\"--type oem system build\\"'
            ' kiwi-no-halt kiwi_version=_9.22.1_'
            ' custom_mount=_/var/tmp/repos_'
            ' sharing_backend=_9p_" '
            '-drive file=system,if=virtio,driver=qcow2,cache=off,snapshot=on '
            '-netdev user,id=user0,hostfwd=tcp::10022-:22 '
            '-device virtio-net-pci,netdev=user0 '
            '-device virtio-serial '
            '-chardev stdio,id=virtiocon0 '
            '-device virtconsole,chardev=virtiocon0 '
            '-fsdev local,security_model=mapped,id=fsdev0,path=desc '
            '-device virtio-9p-pci,id=fs0,fsdev=fsdev0,mount_tag='
            'kiwidescription '
            '-fsdev local,security_model=mapped,id=fsdev1,path=target '
            '-device virtio-9p-pci,id=fs1,fsdev=fsdev1,mount_tag='
            'kiwibundle '
            '-fsdev local,security_model=mapped,id=fsdev2,path=/var/tmp/repos '
            '-device virtio-9p-pci,id=fs2,fsdev=fsdev2,mount_tag='
            'custompath '
            '-initrd initrd '
            '-smp 4'
        )

    @patch('os.environ')
    @patch('os.system')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.box_build.Path.which')
    def test_run_raises_no_qemu_binary_found(
        self, mock_path_which, mock_path_create,
        mock_os_system, mock_os_environ
    ):
        mock_path_which.return_value = None
        with raises(KiwiBoxPluginQEMUBinaryNotFound):
            self.build.run(
                [
                    '--type', 'oem', 'system', 'build',
                    '--description', 'desc', '--target-dir', 'target'
                ],
                keep_open=True,
                kiwi_version='9.22.1',
                custom_shared_path='/var/tmp/repos'
            )

    @patch('os.environ')
    @patch('os.system')
    @patch('subprocess.Popen')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.defaults.Path.which')
    def test_run_with_virtiofs_sharing_raises(
        self, mock_path_which, mock_path_create, mock_subprocess_Popen,
        mock_os_system, mock_os_environ
    ):
        path_which_results = [None, 'qemu-system-x86_64']

        def path_which(name, lookup=None):
            return path_which_results.pop()

        mock_path_which.side_effect = path_which

        self.build.sharing_backend = 'virtiofs'
        with raises(KiwiBoxPluginVirtioFsError):
            self.build.run(
                [
                    '--type', 'oem', 'system', 'build',
                    '--description', 'desc', '--target-dir', 'target'
                ]
            )

        path_which_results = ['virtiofsd', 'qemu-system-x86_64']
        mock_subprocess_Popen.side_effect = Exception
        with raises(KiwiBoxPluginVirtioFsError):
            self.build.run(
                [
                    '--type', 'oem', 'system', 'build',
                    '--description', 'desc', '--target-dir', 'target'
                ]
            )

    @patch('os.environ')
    @patch('os.system')
    @patch('os.path.abspath')
    @patch('subprocess.Popen')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.defaults.Path.which')
    def test_run_with_virtiofs_sharing(
        self, mock_path_which, mock_path_create, mock_subprocess_Popen,
        mock_os_path_abspath, mock_os_system, mock_os_environ
    ):
        def abs_path(arg):
            return 'abspath/{0}'.format(arg)

        def new_process(self, **args):
            return Mock()

        def path_which(name, lookup=None):
            if name == 'qemu-system-x86_64':
                return 'qemu-system-x86_64'
            elif name == 'virtiofsd':
                return '/usr/libexec/virtiofsd'

        mock_path_which.side_effect = path_which
        mock_os_path_abspath.side_effect = abs_path
        mock_subprocess_Popen.side_effect = new_process
        self.build.sharing_backend = 'virtiofs'
        self.build.run(
            [
                '--type', 'oem', 'system', 'build',
                '--description', 'desc', '--target-dir', 'target'
            ],
            keep_open=True,
            kiwi_version='9.22.1',
            custom_shared_path='var/tmp/repos'
        )
        mock_path_create.assert_called_once_with('target')
        mock_os_system.assert_called_once_with(
            'qemu-system-x86_64 '
            '-m 4096 '
            '-accel accel=kvm '
            '-cpu host '
            '-nographic '
            '-nodefaults '
            '-snapshot '
            '-kernel kernel '
            '-append "append kiwi=\\"--type oem system build\\"'
            ' kiwi-no-halt kiwi_version=_9.22.1_'
            ' custom_mount=_var/tmp/repos_'
            ' sharing_backend=_virtiofs_" '
            '-drive file=system,if=virtio,driver=qcow2,cache=off,snapshot=on '
            '-netdev user,id=user0,hostfwd=tcp::10022-:22 '
            '-device virtio-net-pci,netdev=user0 '
            '-device virtio-serial '
            '-chardev stdio,id=virtiocon0 '
            '-device virtconsole,chardev=virtiocon0 '
            '-chardev socket,id=char0,path=/tmp/vhostqemu_0 '
            '-device vhost-user-fs-pci,queue-size=1024,chardev=char0,tag='
            'kiwidescription '
            '-chardev socket,id=char1,path=/tmp/vhostqemu_1 '
            '-device vhost-user-fs-pci,queue-size=1024,chardev=char1,tag='
            'kiwibundle '
            '-chardev socket,id=char2,path=/tmp/vhostqemu_2 '
            '-device vhost-user-fs-pci,queue-size=1024,chardev=char2,tag='
            'custompath '
            '-object '
            'memory-backend-file,id=mem,size=4096,mem-path=/dev/shm,share=on '
            '-numa node,memdev=mem '
            '-initrd initrd '
            '-smp 4'
        )
        assert mock_subprocess_Popen.call_args_list == [
            call(
                [
                    '/usr/libexec/virtiofsd',
                    '--socket-path=/tmp/vhostqemu_0',
                    '-o', 'allow_root',
                    '-o', 'source=abspath/desc',
                    '-o', 'cache=always'
                ], close_fds=True
            ),
            call(
                [
                    '/usr/libexec/virtiofsd',
                    '--socket-path=/tmp/vhostqemu_1',
                    '-o', 'allow_root',
                    '-o', 'source=abspath/target',
                    '-o', 'cache=always'
                ], close_fds=True
            ),
            call(
                [
                    '/usr/libexec/virtiofsd',
                    '--socket-path=/tmp/vhostqemu_2',
                    '-o', 'allow_root',
                    '-o', 'source=abspath/var/tmp/repos',
                    '-o', 'cache=always'
                ], close_fds=True
            )
        ]
        for virtiofsd_process in defaults.VIRTIOFSD_PROCESS_LIST:
            virtiofsd_process.terminate.assert_called_once_with()

    @patch('pwd.getpwuid')
    @patch('time.sleep')
    @patch('os.path.isfile')
    @patch('os.system')
    @patch('kiwi_boxed_plugin.box_build.Command.run')
    @patch('kiwi_boxed_plugin.box_build.Path.create')
    @patch('kiwi_boxed_plugin.defaults.Path.which')
    def test_run_with_sshfs_sharing(
        self, mock_path_which, mock_path_create, mock_Command_run,
        mock_os_system, mock_os_path_isfile, mock_time_sleep,
        mock_pwd_getpwuid
    ):
        def command_run(args, raise_on_error=True):
            if args[0] == 'ssh':
                raise Exception

        def path_which(name, lookup=None):
            if name == 'qemu-system-x86_64':
                return 'qemu-system-x86_64'

        mock_path_which.side_effect = path_which
        mock_Command_run.side_effect = command_run

        mock_pwd_getpwuid.return_value.pw_name = 'user'
        mock_os_path_isfile.return_value = True
        self.build.sharing_backend = 'sshfs'
        self.build.ssh_port = '10022'

        with patch('builtins.open', create=True) as mock_open:
            with patch.dict('os.environ', {'HOME': '~'}):
                file_handle = mock_open.return_value.__enter__.return_value
                file_handle.read.return_value = 'key_type key_value'
                self.build.run(
                    [
                        '--type', 'oem', 'system', 'build',
                        '--description', 'desc', '--target-dir', 'target'
                    ],
                    keep_open=True,
                    kiwi_version='9.22.1',
                    custom_shared_path='var/tmp/repos'
                )

        mock_path_create.assert_called_once_with('target')
        mock_os_system.assert_called_once_with(
            'qemu-system-x86_64 '
            '-m 4096 '
            '-accel accel=kvm '
            '-cpu host '
            '-nographic '
            '-nodefaults '
            '-snapshot '
            '-kernel kernel '
            '-append "append kiwi=\\"--type oem system build\\"'
            ' kiwi-no-halt kiwi_version=_9.22.1_'
            ' custom_mount=_var/tmp/repos_'
            ' ssh_key=_key_value_'
            ' ssh_key_type=_key_type_'
            ' host_kiwidescription=_user@localhost:desc_'
            ' host_kiwibundle=_user@localhost:target_'
            ' host_custompath=_user@localhost:var/tmp/repos_'
            ' sharing_backend=_sshfs_" '
            '-drive file=system,if=virtio,driver=qcow2,cache=off,snapshot=on '
            '-netdev user,id=user0,hostfwd=tcp::10022-:22 '
            '-device virtio-net-pci,netdev=user0 '
            '-device virtio-serial '
            '-chardev stdio,id=virtiocon0 '
            '-device virtconsole,chardev=virtiocon0 '
            '-initrd initrd '
            '-smp 4'
        )
        assert mock_Command_run.call_args_list[0] == call(
            [
                'ssh-keygen', '-R', '[localhost]:10022'
            ], raise_on_error=False
        )
        assert mock_Command_run.call_args_list[1] == call(
            [
                'ssh', '-NT', '-o', 'StrictHostKeyChecking=no',
                'root@localhost', '-p', '10022', '-R',
                '10000:localhost:22'
            ]
        )



import sys

from mock import (
    Mock, patch
)
from kiwi_boxed_plugin.tasks.system_boxbuild import SystemBoxbuildTask


class TestSystemBoxbuildTask:
    def setup(self):
        sys.argv = [
            sys.argv[0],
            '--debug', '--profile', 'foo', '--type', 'oem',
            'system', 'boxbuild',
            '--box', 'suse', '--box-memory', '4', '--box-smp-cpus', '4', '--',
            '--description', '../data/description',
            '--target-dir', '../data/target_dir'
        ]
        self.task = SystemBoxbuildTask()

    def setup_method(self, cls):
        self.setup()

    def _init_command_args(self):
        self.task.command_args = {}
        self.task.command_args['help'] = False
        self.task.command_args['boxbuild'] = False
        self.task.command_args['--list-boxes'] = False
        self.task.command_args['--box'] = None
        self.task.command_args['--box-debug'] = False
        self.task.command_args['--kiwi-version'] = None
        self.task.command_args['--shared-path'] = None
        self.task.command_args['--9p-sharing'] = None
        self.task.command_args['--sshfs-sharing'] = None
        self.task.command_args['--ssh-port'] = '22'
        self.task.command_args['--virtiofs-sharing'] = None
        self.task.command_args['--cpu'] = None
        self.task.command_args['--machine'] = None
        self.task.command_args['<kiwi_build_command_args>'] = [
            '--', '--description', 'foo',
            '--target-dir', 'xxx',
            '--add-package=a', '--add-package', 'b',
            '--allow-existing-root'
        ]

    @patch('kiwi_boxed_plugin.tasks.system_boxbuild.Help')
    def test_process_system_boxbuild_help(self, mock_kiwi_Help):
        Help = Mock()
        mock_kiwi_Help.return_value = Help
        self._init_command_args()
        self.task.command_args['help'] = True
        self.task.command_args['boxbuild'] = True
        self.task.process()
        Help.show.assert_called_once_with(
            'kiwi::system::boxbuild'
        )

    @patch('kiwi_boxed_plugin.tasks.system_boxbuild.PluginConfig')
    def test_process_system_boxbuild_list_boxes(self, mock_PluginConfig):
        plugin = Mock()
        mock_PluginConfig.return_value = plugin
        self._init_command_args()
        self.task.command_args['boxbuild'] = True
        self.task.command_args['--list-boxes'] = True
        self.task.process()
        plugin.dump_config.assert_called_once_with()

    @patch('kiwi_boxed_plugin.tasks.system_boxbuild.BoxBuild')
    def test_process_system_boxbuild(self, mock_BoxBuild):
        self._init_command_args()
        self.task.command_args['boxbuild'] = True
        self.task.command_args['--box'] = 'suse'
        box_build = Mock()
        mock_BoxBuild.return_value = box_build
        self.task.process()
        mock_BoxBuild.assert_called_once_with(
            boxname='suse', ram=None, smp=None, arch='',
            machine=None, cpu='host', sharing_backend='9p',
            ssh_key='id_rsa', ssh_port='22', accel=True
        )
        box_build.run.assert_called_once_with(
            [
                '--debug', '--type', 'oem', '--profile', 'foo',
                'system', 'build',
                '--description', 'foo', '--target-dir', 'xxx',
                '--allow-existing-root',
                '--add-package', 'a', '--add-package', 'b'
            ], True, True, False, None, None
        )

    @patch('kiwi_boxed_plugin.tasks.system_boxbuild.BoxBuild')
    def test_process_system_boxbuild_for_x86_64(self, mock_BoxBuild):
        self._init_command_args()
        self.task.command_args['boxbuild'] = True
        self.task.command_args['--box'] = 'suse'
        self.task.command_args['--x86_64'] = True
        box_build = Mock()
        mock_BoxBuild.return_value = box_build
        self.task.process()
        mock_BoxBuild.assert_called_once_with(
            boxname='suse', ram=None, smp=None, arch='x86_64',
            machine=None, cpu='host', sharing_backend='9p',
            ssh_key='id_rsa', ssh_port='22', accel=True
        )

    @patch('kiwi_boxed_plugin.tasks.system_boxbuild.BoxBuild')
    def test_process_system_boxbuild_for_aarch64(self, mock_BoxBuild):
        self._init_command_args()
        self.task.command_args['boxbuild'] = True
        self.task.command_args['--box'] = 'suse'
        self.task.command_args['--aarch64'] = True
        box_build = Mock()
        mock_BoxBuild.return_value = box_build
        self.task.process()
        mock_BoxBuild.assert_called_once_with(
            boxname='suse', ram=None, smp=None, arch='aarch64',
            machine=None, cpu='host', sharing_backend='9p',
            ssh_key='id_rsa', ssh_port='22', accel=True
        )

    @patch('kiwi_boxed_plugin.tasks.system_boxbuild.BoxBuild')
    def test_process_system_boxbuild_with_sharing_backend(self, mock_BoxBuild):
        self._init_command_args()
        self.task.command_args['boxbuild'] = True
        self.task.command_args['--box'] = 'suse'
        self.task.command_args['--9p-sharing'] = True
        box_build = Mock()
        mock_BoxBuild.return_value = box_build
        self.task.process()
        mock_BoxBuild.assert_called_once_with(
            boxname='suse', ram=None, smp=None, arch='',
            machine=None, cpu='host', sharing_backend='9p',
            ssh_key='id_rsa', ssh_port='22', accel=True
        )
        self.task.command_args['--9p-sharing'] = False
        self.task.command_args['--virtiofs-sharing'] = True
        mock_BoxBuild.reset_mock()
        self.task.process()
        mock_BoxBuild.assert_called_once_with(
            boxname='suse', ram=None, smp=None, arch='',
            machine=None, cpu='host', sharing_backend='virtiofs',
            ssh_key='id_rsa', ssh_port='22', accel=True
        )
        self.task.command_args['--9p-sharing'] = False
        self.task.command_args['--virtiofs-sharing'] = False
        self.task.command_args['--sshfs-sharing'] = True
        mock_BoxBuild.reset_mock()
        self.task.process()
        mock_BoxBuild.assert_called_once_with(
            boxname='suse', ram=None, smp=None, arch='',
            machine=None, cpu='host', sharing_backend='sshfs',
            ssh_key='id_rsa', ssh_port='22', accel=True
        )



from pkg_resources import resource_filename
from kiwi_boxed_plugin.defaults import Defaults
from mock import patch
import os


class MockedPath:
    def __init__(self):
        self.p: str | None = None

    def home(self):
        return self

    @staticmethod
    def exists():
        return True

    def joinpath(self, f: str):
        self.p = os.path.join("/home/zoidberg", f)
        return self

    def as_posix(self):
        return self.p


class TestDefaults:
    def test_get_plugin_config_file(self):
        assert Defaults.get_plugin_config_file() == resource_filename(
            'kiwi_boxed_plugin', 'config/kiwi_boxed_plugin.yml'
        )

    @patch("os.path.exists", lambda f: True)
    @patch.dict(os.environ, KIWI_BOXED_PLUGIN_CFG="aarchderwelt.conf")
    def test_get_plugin_config_file_env(self):
        assert Defaults.get_plugin_config_file() == "aarchderwelt.conf", \
            "aarch64 aonfiguration from the environment variable do not match"

    @patch("os.path.abspath", lambda f: "/highway/to/hell.conf")
    @patch("os.path.exists", lambda f: True)
    def test_get_plugin_config_file_currdir(self):
        assert Defaults.get_plugin_config_file() == "/highway/to/hell.conf", \
            "Should contain absolute path to the config"

    @patch("pathlib.Path", MockedPath())
    def test_get_plugin_config_file_local_kiwi(self):
        assert Defaults.get_plugin_config_file() == "/home/zoidberg/.config/kiwi/kiwi_boxed_plugin.yml", \
            "Should contain local Kiwi Box config"

    @patch("os.path.exists", lambda f: f == "/etc/kiwi_boxed_plugin.yml")
    def test_get_plugin_config_file_etc(self):
        assert Defaults.get_plugin_config_file() == "/etc/kiwi_boxed_plugin.yml", \
            "Should contain Kiwi Box config in /etc dir"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# KIWI NG documentation build configuration file
#
import sys
from os.path import abspath, dirname, join, normpath
import shlex
import sphinx_rtd_theme

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
_path = normpath(join(dirname(__file__), "../.."))
sys.path.insert(0, _path)


# -- General configuration ------------------------------------------------

extensions = [
    'sphinx.ext.extlinks',
    'sphinx.ext.viewcode',
    'sphinx.ext.intersphinx'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['.templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The encoding of source files.
source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

default_role="py:obj"

# General information about the project.
project = 'KIWI - Boxed Build Plugin'
copyright = '2020, Marcus Schäfer'
author = 'Marcus Schäfer'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '0.2.31'
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

autosummary_generate = True

# -- Options for HTML output ----------------------------------------------

html_sidebars = {
   '**': [
          'localtoc.html', 'relations.html',
          'about.html', 'searchbox.html',
         ]
}

html_theme = "sphinx_rtd_theme"

html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

html_theme_options = {
    'collapse_navigation': False,
    'display_version': False
}

# -- Options for manual page output ---------------------------------------

# The man page toctree documents.
system_boxbuild_doc = 'commands/system_boxbuild'

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        system_boxbuild_doc,
        'kiwi::system::boxbuild',
        'Self Contained Image Build',
        [author],
        8
    )
]

intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}



import io
from mock import (
    patch, Mock, MagicMock, call
)

from kiwi_boxed_plugin.box_download import (
    BoxDownload, vm_setup_type
)


class TestBoxDownload:
    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    @patch('kiwi_boxed_plugin.box_download.Path')
    @patch('kiwi_boxed_plugin.box_download.DirFiles')
    def setup(self, mock_DirFiles, mock_Path, mock_get_plugin_config_file):
        self.box_stage = Mock()
        self.box_stage.register.return_value = 'register_file'
        mock_DirFiles.return_value = self.box_stage
        mock_get_plugin_config_file.return_value = \
            '../data/kiwi_boxed_plugin.yml'
        with patch.dict('os.environ', {'HOME': 'HOME'}):
            self.box = BoxDownload('suse', 'x86_64')
        mock_Path.create.assert_called_once_with(
            'HOME/.kiwi_boxes/suse'
        )
        self.result = vm_setup_type(
            system='HOME/.kiwi_boxes/suse/'
            'SUSE-Box.x86_64-1.42.1-System-BuildBox.qcow2',
            kernel='HOME/.kiwi_boxes/suse/kernel.x86_64',
            initrd='HOME/.kiwi_boxes/suse/initrd.x86_64',
            append='console=hvc0 root=/dev/vda1 rd.plymouth=0',
            ram=8096,
            smp=4
        )

    @patch('kiwi_boxed_plugin.defaults.Defaults.get_plugin_config_file')
    @patch('kiwi_boxed_plugin.box_download.Path')
    @patch('kiwi_boxed_plugin.box_download.DirFiles')
    def setup_method(
        self, cls, mock_DirFiles, mock_Path, mock_get_plugin_config_file
    ):
        self.setup()

    @patch('kiwi_boxed_plugin.box_download.Command.run')
    @patch('kiwi_boxed_plugin.box_download.Uri')
    @patch('kiwi_boxed_plugin.box_download.SolverRepository.new')
    @patch('kiwi_boxed_plugin.box_download.Checksum')
    @patch('os.path.exists')
    @patch('os.chdir')
    @patch('kiwi_boxed_plugin.box_download.FetchFiles')
    def test_fetch_checksum_did_not_match(
        self, mock_FetchFiles, mock_os_chdir, mock_os_path_exist,
        mock_Checksum, mock_SolverRepository, mock_Uri, mock_Command_run
    ):
        fetcher = Mock()
        mock_FetchFiles.return_value = fetcher
        checksum = Mock()
        checksum.matches.return_value = False
        checksum.sha256.return_value = 'sum'
        mock_Checksum.return_value = checksum
        repo = Mock()
        repo._get_mime_typed_uri.return_value = 'uri://'
        mock_SolverRepository.return_value = repo
        mock_os_path_exist.return_value = False
        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value
            with patch.dict('os.environ', {'HOME': 'HOME'}):
                assert self.box.fetch(update_check=True) == self.result
            assert self.box_stage.register.call_args_list == [
                call(
                    'HOME/.kiwi_boxes/suse/'
                    'SUSE-Box.x86_64-1.42.1-System-BuildBox.report'
                ),
                call(
                    'HOME/.kiwi_boxes/suse/'
                    'SUSE-Box.x86_64-1.42.1-System-BuildBox.report.sha256'
                ),
                call(
                    'HOME/.kiwi_boxes/suse/'
                    'SUSE-Box.x86_64-1.42.1-Kernel-BuildBox.tar.xz'
                ),
                call(
                    'HOME/.kiwi_boxes/suse/'
                    'SUSE-Box.x86_64-1.42.1-System-BuildBox.qcow2'
                )
            ]
            mock_open.assert_called_once_with(
                self.box_stage.register.return_value, 'w'
            )
            file_handle.write.assert_called_once_with('sum')
            repo.download_from_repository.assert_called_once_with(
                'SUSE-Box.x86_64-1.42.1-System-BuildBox.report',
                self.box_stage.register.return_value
            )
            assert fetcher.wget.call_args_list == [
                call(
                    url='uri:///SUSE-Box.x86_64-1.42.1-Kernel-BuildBox.tar.xz',
                    filename='register_file'
                ),
                call(
                    url='uri:///SUSE-Box.x86_64-1.42.1-System-BuildBox.qcow2',
                    filename='register_file'
                )
            ]
            self.box_stage.commit.assert_called_once_with()
            assert mock_Command_run.call_args_list == [
                call(
                    [
                        'tar', '-C', 'HOME/.kiwi_boxes/suse',
                        '--transform', 's/.*/kernel.x86_64/',
                        '--wildcards', '-xf',
                        'HOME/.kiwi_boxes/suse/'
                        'SUSE-Box.x86_64-1.42.1-Kernel-BuildBox.tar.xz',
                        '*.kernel'
                    ]
                ),
                call(
                    [
                        'tar', '-C', 'HOME/.kiwi_boxes/suse',
                        '--transform', 's/.*/initrd.x86_64/',
                        '--wildcards', '-xf',
                        'HOME/.kiwi_boxes/suse/'
                        'SUSE-Box.x86_64-1.42.1-Kernel-BuildBox.tar.xz',
                        '*.initrd'
                    ]
                )
            ]

    @patch('kiwi_boxed_plugin.box_download.Command.run')
    @patch('kiwi_boxed_plugin.box_download.Uri')
    @patch('kiwi_boxed_plugin.box_download.SolverRepository.new')
    @patch('kiwi_boxed_plugin.box_download.Checksum')
    @patch('os.path.exists')
    def test_fetch_checksum_matches(
        self, mock_os_path_exist, mock_Checksum, mock_SolverRepository,
        mock_Uri, mock_Command_run
    ):
        checksum = Mock()
        checksum.matches.return_value = True
        checksum.sha256.return_value = 'sum'
        mock_Checksum.return_value = checksum
        repo = Mock()
        mock_SolverRepository.return_value = repo
        mock_os_path_exist.return_value = True
        assert self.box.fetch(update_check=True) == self.result
        repo.download_from_repository.assert_called_once_with(
            'SUSE-Box.x86_64-1.42.1-System-BuildBox.report',
            self.box_stage.register.return_value
        )

    @patch('kiwi_boxed_plugin.box_download.Command.run')
    @patch('kiwi_boxed_plugin.box_download.Uri')
    @patch('kiwi_boxed_plugin.box_download.SolverRepository.new')
    @patch('kiwi_boxed_plugin.box_download.Checksum')
    @patch('os.path.exists')
    def test_fetch_update_check_disabled(
        self, mock_os_path_exist, mock_Checksum, mock_SolverRepository,
        mock_Uri, mock_Command_run
    ):
        mock_os_path_exist.return_value = True
        assert self.box.fetch(update_check=False) == self.result
